package top.huic.trtc_cloud_plugin;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

import androidx.annotation.NonNull;

import com.google.gson.Gson;
import com.tencent.liteav.audio.TXAudioEffectManager;
import com.tencent.liteav.beauty.TXBeautyManager;
import com.tencent.liteav.device.TXDeviceManager;
import com.tencent.trtc.TRTCCloud;
import com.tencent.trtc.TRTCCloudDef;
import com.tencent.trtc.TRTCCloudListener;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.platform.PlatformViewRegistry;
import top.huic.trtc_cloud_plugin.listener.CustomTRTCCloudListener;
import top.huic.trtc_cloud_plugin.util.CommonUtil;
import top.huic.trtc_cloud_plugin.util.GenerateTestUserSig;
import top.huic.trtc_cloud_plugin.view.TRTCCloudVideoPlatformView;

/**
 * 安卓中间层-腾讯云视频通话功能的主要接口类
 */
public class TRTCCloudPlugin implements FlutterPlugin, MethodCallHandler {
    /**
     * methodChannel标识
     */
    private static final String CHANNEL_SIGN = "trtcCloudChannel";
    
    private TRTCCloud trtcCloud;
    private Context trtcContext;
    private TXDeviceManager txDeviceManager;
    private TXBeautyManager txBeautyManager;
    private TXAudioEffectManager txAudioEffectManager;
    private CustomTRTCCloudListener trtcListener;

    public TRTCCloudPlugin() {
    }

    private TRTCCloudPlugin(BinaryMessenger messenger, Context context, MethodChannel channel, PlatformViewRegistry registry) {
        trtcContext = context;
        trtcListener = new CustomTRTCCloudListener(channel);
        // 注册View
        registry.registerViewFactory(TRTCCloudVideoPlatformView.SIGN, new TRTCCloudVideoPlatformView(context, messenger));
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        final MethodChannel channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), CHANNEL_SIGN);
        channel.setMethodCallHandler(new TRTCCloudPlugin(flutterPluginBinding.getBinaryMessenger(), flutterPluginBinding.getApplicationContext(), channel, flutterPluginBinding.getPlatformViewRegistry()));
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    }

    public static void registerWith(Registrar registrar) {
        if (registrar.activity() == null) {
            return;
        }
        final MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL_SIGN);
        channel.setMethodCallHandler(new TRTCCloudPlugin(registrar.messenger(), registrar.context(), channel, registrar.platformViewRegistry()));
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        try {
            Method method = TRTCCloudPlugin.class.getDeclaredMethod(call.method,MethodCall.class,Result.class);
            method.invoke(this,call,result);
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
    }
    /**
     * 创建 TRTCCloud 单例
     */
    private void sharedInstance(@NonNull MethodCall call, @NonNull Result result) {
        // 初始化实例
        trtcCloud = TRTCCloud.sharedInstance(trtcContext);
        trtcCloud.setListener(trtcListener);
        result.success(null);
    }
    /**
     * 销毁 TRTCCloud 单例
     */
    private void destroySharedInstance(@NonNull MethodCall call, @NonNull Result result) {
        TRTCCloud.destroySharedInstance();
        trtcCloud = null;
        result.success(null);
    }
    /**
     * 进入房间
     */
    private void enterRoom(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String param = CommonUtil.getParam(call, result, "param");
        int scene = CommonUtil.getParam(call, result, "scene");
        trtcCloud.enterRoom(new Gson().fromJson(param, TRTCCloudDef.TRTCParams.class), scene);
        result.success(null);
    }
    
    /**
     * 退出房间
     */
    private void exitRoom(@NonNull MethodCall call, @NonNull Result result) {
        trtcCloud.exitRoom();
        result.success(null);
    }

    /**
     * 切换角色
     */
    private void switchRole(@NonNull MethodCall call, @NonNull Result result) {
        int role = CommonUtil.getParam(call, result, "role");
        trtcCloud.switchRole(role);
        result.success(null);
    }

    /**
     * 设置音视频数据接收模式（需要在进房前设置才能生效）。
     */
    private void setDefaultStreamRecvMode(@NonNull MethodCall call, @NonNull Result result) {
        boolean autoRecvAudio = CommonUtil.getParam(call, result, "autoRecvAudio");
        boolean autoRecvVideo = CommonUtil.getParam(call, result, "autoRecvVideo");
        trtcCloud.setDefaultStreamRecvMode(autoRecvAudio, autoRecvVideo);
        result.success(null);
    }
    /**
     * 设置云端的混流转码
     */
    private void setMixTranscodingConfig(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String config = CommonUtil.getParam(call, result, "config");
        trtcCloud.setMixTranscodingConfig(new Gson().fromJson(config, TRTCCloudDef.TRTCTranscodingConfig.class));
        result.success(null);
    }
    /**
     * 停止本地视频采集
     */
    private void stopLocalPreview(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        trtcCloud.stopLocalPreview();
        result.success(null);
    }
    /**
     * 停止远程显示
     */
    private void stopRemoteView(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = CommonUtil.getParam(call, result, "userId");
        int streamType = CommonUtil.getParam(call, result, "streamType");
        trtcCloud.stopRemoteView(userId, streamType);
        result.success(null);
    }
    /**
     * 停止远端辅流显示
     */
    private void stopRemoteSubStreamView(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = CommonUtil.getParam(call, result, "userId");
        trtcCloud.stopRemoteSubStreamView(userId);
        result.success(null);
    }
    /**
     * 停止所有远程显示
     */
    private void stopAllRemoteView(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        trtcCloud.stopAllRemoteView();
        result.success(null);
    }
    /**
     * 静音 / 取消静音
     */
    private void muteRemoteAudio(@NonNull MethodCall call, @NonNull Result result) {
        String userId = CommonUtil.getParam(call, result, "userId");
        boolean mute = CommonUtil.getParam(call, result, "mute");
        trtcCloud.muteRemoteAudio(userId, mute);
        result.success(null);
    }

    /**
     * 静音 / 取消静音 所有用户
     */
    private void muteAllRemoteAudio(@NonNull MethodCall call, @NonNull Result result) {
        boolean mute = CommonUtil.getParam(call, result, "mute");
        trtcCloud.muteAllRemoteAudio(mute);
        result.success(null);
    }

    /**
     * 静音 / 取消静音
     */
    private void setRemoteAudioVolume(@NonNull MethodCall call, @NonNull Result result) {
        String userId = CommonUtil.getParam(call, result, "userId");
        int volume = CommonUtil.getParam(call, result, "volume");
        trtcCloud.setRemoteAudioVolume(userId, volume);
        result.success(null);
    }

    /**
     * 设置 SDK 采集音量。
     */
    private void setAudioCaptureVolume(@NonNull MethodCall call, @NonNull Result result) {
        int volume = CommonUtil.getParam(call, result, "volume");
        trtcCloud.setAudioCaptureVolume(volume);
        result.success(null);
    }

    /**
     * 获取 SDK 采集音量。
     */
    private void getAudioCaptureVolume(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(trtcCloud.getAudioCaptureVolume());
    }

    /**
     * 设置 SDK 播放音量。
     */
    private void setAudioPlayoutVolume(@NonNull MethodCall call, @NonNull Result result) {
        int volume = CommonUtil.getParam(call, result, "volume");
        trtcCloud.setAudioPlayoutVolume(volume);
        result.success(null);
    }

    /**
     * 获取 SDK 播放音量。
     */
    private void getAudioPlayoutVolume(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(trtcCloud.getAudioPlayoutVolume());
    }

    /**
     * 开启本地音频采集，并设置音频质量 主播端的音质越高，观众端的听感越好，但传输所依赖的带宽也就越高，在带宽有限的场景下也更容易出现卡顿。
     */
    private void startLocalAudio(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int quality = CommonUtil.getParam(call, result, "quality");
        trtcCloud.startLocalAudio(quality);
        result.success(null);
    }
    /**
     * 关闭本地音频采集
     */
    private void stopLocalAudio(@NonNull MethodCall call, @NonNull Result result) {
        trtcCloud.stopLocalAudio();
        result.success(null);
    }

    /**
     * 暂停接收指定的远端视频流。
     */
    private void muteRemoteVideoStream(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = CommonUtil.getParam(call, result, "userId");
        boolean mute = CommonUtil.getParam(call, result, "mute");
        trtcCloud.muteRemoteVideoStream(userId, mute);
        result.success(null);
    }

    /**
     * 停止接收所有远端视频流。
     */
    private void muteAllRemoteVideoStreams(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        boolean mute = CommonUtil.getParam(call, result, "mute");
        trtcCloud.muteAllRemoteVideoStreams(mute);
        result.success(null);
    }

    /**
     * 设置视频编码相关。
     */
    private void setVideoEncoderParam(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String param = CommonUtil.getParam(call, result, "param");
        trtcCloud.setVideoEncoderParam(new Gson().fromJson(param, TRTCCloudDef.TRTCVideoEncParam.class));
        result.success(null);
    }

    /**
     * 设置网络流控相关参数。
     */
    private void setNetworkQosParam(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String param = CommonUtil.getParam(call, result, "param");
        trtcCloud.setNetworkQosParam(new Gson().fromJson(param, TRTCCloudDef.TRTCNetworkQosParam.class));
        result.success(null);
    }
    /**
     * 设置本地图像的渲染模式。
     */
    private void setLocalRenderParams(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String param = CommonUtil.getParam(call, result, "param");
        trtcCloud.setLocalRenderParams(new Gson().fromJson(param, TRTCCloudDef.TRTCRenderParams.class));
        result.success(null);
    }

    /**
     * 设置本地图像的渲染模式。
     */
    private void setRemoteRenderParams(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = CommonUtil.getParam(call, result, "userId");
        int streamType = CommonUtil.getParam(call, result, "streamType");
        String param = CommonUtil.getParam(call, result, "param");
        trtcCloud.setRemoteRenderParams(userId, streamType,new Gson().fromJson(param, TRTCCloudDef.TRTCRenderParams.class));
        result.success(null);
    }

    /**
     * 设置视频编码输出的（也就是远端用户观看到的，以及服务器录制下来的）画面方向。
     */
    private void setVideoEncoderRotation(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int rotation = CommonUtil.getParam(call, result, "rotation");
        trtcCloud.setVideoEncoderRotation(rotation);
        result.success(null);
    }

    /**
     * 设置编码器输出的画面镜像模式。
     */
    private void setVideoEncoderMirror(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        boolean mirror = CommonUtil.getParam(call, result, "mirror");
        trtcCloud.setVideoEncoderMirror(mirror);
        result.success(null);
    }

    /**
     * 设置重力感应的适应模式。
     */
    private void setGSensorMode(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int mode = CommonUtil.getParam(call, result, "mode");
        trtcCloud.setGSensorMode(mode);
        result.success(null);
    }

    /**
     * 开启大小画面双路编码模式。
     */
    private void enableEncSmallVideoStream(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        boolean enable = CommonUtil.getParam(call, result, "enable");
        String smallVideoEncParam = CommonUtil.getParam(call, result, "smallVideoEncParam");
        trtcCloud.enableEncSmallVideoStream(enable, new Gson().fromJson(smallVideoEncParam, TRTCCloudDef.TRTCVideoEncParam.class));
        result.success(null);
    }

    /**
     * 选定观看指定 uid 的大画面或小画面。
     */
    private void setRemoteVideoStreamType(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = CommonUtil.getParam(call, result, "userId");
        int streamType = CommonUtil.getParam(call, result, "streamType");
        trtcCloud.setRemoteVideoStreamType(userId, streamType);
        result.success(null);
    }

    /**
     * 设定观看方优先选择的视频质量。
     */
    private void setPriorRemoteVideoStreamType(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int streamType = CommonUtil.getParam(call, result, "streamType");
        trtcCloud.setPriorRemoteVideoStreamType(streamType);
        result.success(null);
    }
    /**
     * 视频画面截图
     */
    private void snapshotVideo(@NonNull MethodCall call, @NonNull final MethodChannel.Result result) {
        String userId = CommonUtil.getParamCanBeNull(call, result, "userId");
        int streamType = CommonUtil.getParam(call, result, "streamType");
        final String path = CommonUtil.getParam(call, result, "path");
        trtcCloud.snapshotVideo(userId, streamType, new TRTCCloudListener.TRTCSnapshotListener() {
            @Override
            public void onSnapshotComplete(Bitmap bitmap) {
                System.out.println(bitmap);
                try {
                    String[] pathArr = path.split("\\.");
                    Bitmap.CompressFormat bitComp = Bitmap.CompressFormat.PNG;
                    if(pathArr[pathArr.length-1].equals("jpg")) {
                        bitComp = Bitmap.CompressFormat.JPEG;
                    } else if (pathArr[pathArr.length-1].equals("webp")) {
                        bitComp = Bitmap.CompressFormat.WEBP;
                    }
                    FileOutputStream fos = new FileOutputStream(path);
                    boolean isSuccess = bitmap.compress(bitComp, 100, fos);
                    if(isSuccess) {
                        trtcListener.onSnapshotComplete("success", path);
                    } else {
                        trtcListener.onSnapshotComplete("bitmap compress failed", "");
                    }

                } catch (FileNotFoundException e) {
                    trtcListener.onSnapshotComplete(e.toString(), "");
                }
            }
        });

        result.success(null);
    }

    /**
     * 静音本地的音频。
     */
    private void muteLocalAudio(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        boolean mute = CommonUtil.getParam(call, result, "mute");
        trtcCloud.muteLocalAudio(mute);
        result.success(null);
    }

    /**
     * 关闭本地的视频。
     */
    private void muteLocalVideo(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        boolean mute = CommonUtil.getParam(call, result, "mute");
        trtcCloud.muteLocalVideo(mute);
        result.success(null);
    }

    /**
     * 设置音频路由。
     */
    private void setAudioRoute(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int route = CommonUtil.getParam(call, result, "route");
        trtcCloud.setAudioRoute(route);
        result.success(null);
    }

    /**
     * 启用音量大小提示。
     */
    private void enableAudioVolumeEvaluation(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int intervalMs = CommonUtil.getParam(call, result, "intervalMs");
        trtcCloud.enableAudioVolumeEvaluation(intervalMs);
        result.success(null);
    }

    /**
     * 开始录音。
     */
    private void startAudioRecording(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        TRTCCloudDef.TRTCAudioRecordingParams params = new TRTCCloudDef.TRTCAudioRecordingParams();
        params.filePath = CommonUtil.getParam(call, result, "filePath");
        trtcCloud.startAudioRecording(params);
        result.success(null);
    }

    /**
     * 停止录音。
     */
    private void stopAudioRecording(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        trtcCloud.stopAudioRecording();
        result.success(null);
    }

    /**
     * 设置通话时使用的系统音量类型。
     */
    private void setSystemVolumeType(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int type = CommonUtil.getParam(call, result, "type");
        trtcCloud.setSystemVolumeType(type);
        result.success(null);
    }

    /**
     * 开启耳返。
     */
    private void enableAudioEarMonitoring(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        boolean enable = CommonUtil.getParam(call, result, "enable");
        trtcCloud.enableAudioEarMonitoring(enable);
        result.success(null);
    }
    /**
     * 切换摄像头。
     */
    private void isFrontCamera(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(txDeviceManager.isFrontCamera());
    }
    /**
     * 切换摄像头。
     */
    private void switchCamera(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        txDeviceManager.switchCamera();
        result.success(null);
    }

    /**
     * 查询当前摄像头是否支持缩放。
     */
    private void isCameraZoomSupported(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(txDeviceManager.isCameraZoomSupported());
    }

    /**
     * 获取摄像头的缩放因子。
     */
    private void getCameraZoomMaxRatio(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(txDeviceManager.getCameraZoomMaxRatio());
    }

    /**
     * 设置摄像头缩放因子（焦距）。
     */
    private void setCameraZoomRatio(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int value = CommonUtil.getParam(call, result, "value");
        txDeviceManager.setCameraZoomRatio(value);
        result.success(null);
    }

    /**
     * 查询是否支持开关闪光灯（手电筒模式）。
     */
    private void isCameraTorchSupported(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(txDeviceManager.isCameraTorchSupported());
    }

    /**
     * 开启闪光灯
     */
    private void enableCameraTorch(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        boolean enable = CommonUtil.getParam(call, result, "enable");
        txDeviceManager.enableCameraTorch(enable);
        result.success(null);
    }

    /**
     * 查询是否支持设置焦点。
     */
    private void isCameraFocusPositionInPreviewSupported(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(txDeviceManager.isCameraFocusPositionInPreviewSupported());
    }

    /**
     * 设置摄像头焦点。
     */
    private void setCameraFocusPosition(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int x = CommonUtil.getParam(call, result, "x");
        int y = CommonUtil.getParam(call, result, "y");
        txDeviceManager.setCameraFocusPosition(x, y);
        result.success(null);
    }

    /**
     * 查询是否支持自动识别人脸位置。
     */
    private void isCameraAutoFocusFaceModeSupported(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(txDeviceManager.isCameraAutoFocusFaceModeSupported());
    }

    //获取美颜管理对象
    private void getDeviceManager(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        txDeviceManager = trtcCloud.getDeviceManager();
    }

    //获取美颜管理对象
    private void getBeautyManager(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        txBeautyManager = trtcCloud.getBeautyManager();
    }

    //获取音效管理类 TXAudioEffectManager
    private void getAudioEffectManager(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        txAudioEffectManager = trtcCloud.getAudioEffectManager();
    }

    /**
     * 添加水印
     */
    private void setWatermark(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String imageUrl = CommonUtil.getParam(call, result, "imageUrl");
        int streamType = CommonUtil.getParam(call, result, "streamType");
        String xStr = CommonUtil.getParam(call, result, "x");
        float x = Float.parseFloat(xStr);
        String yStr = CommonUtil.getParam(call, result, "y");
        float y = Float.parseFloat(yStr);
        String widthStr = CommonUtil.getParam(call, result, "width");
        float width = Float.parseFloat(widthStr);
        trtcCloud.setWatermark(BitmapFactory.decodeFile(imageUrl), streamType, x, y, width);
        result.success(null);
    }

    /**
     * 开始进行网络测速（视频通话期间请勿测试，以免影响通话质量）
     */
    private void startSpeedTest(@NonNull MethodCall call, @NonNull Result result) {
        int sdkAppId = CommonUtil.getParam(call, result, "sdkAppId");
        String userId = CommonUtil.getParam(call, result, "userId");
        String userSig = CommonUtil.getParam(call, result, "userSig");
        trtcCloud.startSpeedTest(sdkAppId,userId,userSig);
        result.success(null);
    }

    /**
     * 停止服务器测速
     */
    private void stopSpeedTest(@NonNull MethodCall call, @NonNull Result result) {
        trtcCloud.stopSpeedTest();
        result.success(null);
    }

    /**
     * 获取 SDK 版本信息
     */
    private void getSDKVersion(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        result.success(trtcCloud.getSDKVersion());
    }

    /**
     * 设置 Log 输出级别
     */
    private void setLogLevel(@NonNull MethodCall call, @NonNull Result result) {
        int level = CommonUtil.getParam(call, result, "level");
        trtcCloud.setLogLevel(level);
        result.success(null);
    }
    /**
     * 设置日志显示
     */
    private void setConsoleEnabled(@NonNull MethodCall call, @NonNull Result result) {
        boolean enabled = CommonUtil.getParam(call, result, "enabled");
        TRTCCloud.setConsoleEnabled(enabled);
        result.success(null);
    }
    /**
     * 设置Debug视图
     */
    private void showDebugView(@NonNull MethodCall call, @NonNull Result result) {
        int mode = CommonUtil.getParam(call, result, "mode");
        trtcCloud.showDebugView(mode);
        result.success(null);
    }


    /**
     * 设置美颜类型
     */
    private void setBeautyStyle(@NonNull MethodCall call, @NonNull Result result) {
        int beautyStyle = CommonUtil.getParam(call, result, "beautyStyle");
        txBeautyManager.setBeautyStyle(beautyStyle);
        result.success(null);
    }
    /**
     * 设置指定素材滤镜特效
     */
    private void setFilter(@NonNull MethodCall call, @NonNull Result result) {
        String imageUrl = CommonUtil.getParam(call, result, "imageUrl");
        txBeautyManager.setFilter(BitmapFactory.decodeFile(imageUrl));
        result.success(null);
    }
    /**
     * 设置滤镜浓度
     */
    private void setFilterStrength(@NonNull MethodCall call, @NonNull Result result) {
        float strength = CommonUtil.getParam(call, result, "strength");
        txBeautyManager.setFilterStrength(strength);
        result.success(null);
    }
    /**
     * 设置美颜级别
     */
    private void setBeautyLevel(@NonNull MethodCall call, @NonNull Result result) {
        int beautyLevel = CommonUtil.getParam(call, result, "beautyLevel");
        txBeautyManager.setBeautyLevel(beautyLevel);
        result.success(null);
    }
    /**
     * 设置美白级别
     */
    private void setWhitenessLevel(@NonNull MethodCall call, @NonNull Result result) {
        int whitenessLevel = CommonUtil.getParam(call, result, "whitenessLevel");
        txBeautyManager.setWhitenessLevel(whitenessLevel);
        result.success(null);
    }
    /**
     * 开启清晰度增强
     */
    private void enableSharpnessEnhancement(@NonNull MethodCall call, @NonNull Result result) {
        boolean enable = CommonUtil.getParam(call, result, "enable");
        txBeautyManager.enableSharpnessEnhancement(enable);
        result.success(null);
    }
    /**
     * 设置红润级别
     */
    private void setRuddyLevel(@NonNull MethodCall call, @NonNull Result result) {
        int ruddyLevel = CommonUtil.getParam(call, result, "ruddyLevel");
        txBeautyManager.setRuddyLevel(ruddyLevel);
        result.success(null);
    }

    /**
     * 开启耳返
     */
    private void enableVoiceEarMonitor(@NonNull MethodCall call, @NonNull Result result) {
        boolean enable = CommonUtil.getParam(call, result, "enable");
        txAudioEffectManager.enableVoiceEarMonitor(enable);
        result.success(null);
    }
    /**
     * 设置耳返音量。
     */
    private void setVoiceEarMonitorVolume(@NonNull MethodCall call, @NonNull Result result) {
        int volume = CommonUtil.getParam(call, result, "volume");
        txAudioEffectManager.setVoiceEarMonitorVolume(volume);
        result.success(null);
    }
    /**
     * 设置人声的混响效果（KTV、小房间、大会堂、低沉、洪亮...）
     */
    private void setVoiceReverbType(@NonNull MethodCall call, @NonNull Result result) {
        int type = CommonUtil.getParam(call, result, "type");
        TXAudioEffectManager.TXVoiceReverbType reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_0;
        switch (type) {
            case 0:
                reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_0;
                break;
            case 1:
                reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_1;
                break;
            case 2:
                reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_2;
                break;
            case 3:
                reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_3;
                break;
            case 4:
                reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_4;
                break;
            case 5:
                reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_5;
                break;
            case 6:
                reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_6;
                break;
            case 7:
                reverbType = TXAudioEffectManager.TXVoiceReverbType.TXLiveVoiceReverbType_7;
                break;
        }
        txAudioEffectManager.setVoiceReverbType(reverbType);
        result.success(null);
    }
    /**
     * 设置人声的变声特效（萝莉、大叔、重金属、外国人...）
     */
    private void setVoiceChangerType(@NonNull MethodCall call, @NonNull Result result) {
        int type = CommonUtil.getParam(call, result, "type");
        TXAudioEffectManager.TXVoiceChangerType changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_0;
        switch (type) {
            case 0:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_0;
                break;
            case 1:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_1;
                break;
            case 2:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_2;
                break;
            case 3:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_3;
                break;
            case 4:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_4;
                break;
            case 5:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_5;
                break;
            case 6:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_6;
                break;
            case 7:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_7;
                break;
            case 8:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_8;
                break;
            case 9:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_9;
                break;
            case 10:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_10;
                break;
            case 11:
                changerType = TXAudioEffectManager.TXVoiceChangerType.TXLiveVoiceChangerType_11;
                break;
        }
        txAudioEffectManager.setVoiceChangerType(changerType);
        result.success(null);
    }
    /**
     * 设置麦克风采集人声的音量
     */
    private void setVoiceCaptureVolume(@NonNull MethodCall call, @NonNull Result result) {
        int volume = CommonUtil.getParam(call, result, "volume");
        txAudioEffectManager.setVoiceCaptureVolume(volume);
        result.success(null);
    }

    /**
     * 设置背景音乐的播放进度回调接口
     */
    private void setMusicObserver(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        txAudioEffectManager.setMusicObserver(id, new TXAudioEffectManager.TXMusicPlayObserver() {
            @Override
            public void onStart(int i, int i1) {
                trtcListener.onMusicObserverStart(i, i1);
            }

            @Override
            public void onPlayProgress(int i, long l, long l1) {
                trtcListener.onMusicObserverPlayProgress(i, l,l1);
            }

            @Override
            public void onComplete(int i, int i1) {
                trtcListener.onMusicObserverComplete(i, i1);
            }
        });
        result.success(null);
    }

    /**
     * 开始播放背景音乐
     */
    private void startPlayMusic(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        String path = CommonUtil.getParam(call, result, "path");
        TXAudioEffectManager.AudioMusicParam audioMusicParam = new TXAudioEffectManager.AudioMusicParam(id, path);
        int loopCount = CommonUtil.getParam(call, result, "loopCount");
        boolean publish = CommonUtil.getParam(call, result, "publish");
        boolean isShortFile = CommonUtil.getParam(call, result, "isShortFile");
        String startTimeMSParam= CommonUtil.getParam(call, result, "startTimeMS");
        String endTimeMSParam= CommonUtil.getParam(call, result, "endTimeMS");
        long startTimeMS = Long.parseLong(startTimeMSParam);
        long endTimeMS = Long.parseLong(endTimeMSParam);
        audioMusicParam.loopCount = loopCount;
        audioMusicParam.publish = publish;
        audioMusicParam.isShortFile = isShortFile;
        audioMusicParam.startTimeMS = startTimeMS;
        audioMusicParam.endTimeMS = endTimeMS;

        boolean isSuccess = txAudioEffectManager.startPlayMusic(audioMusicParam);
        result.success(isSuccess);
    }

    /**
     * 停止播放背景音乐
     */
    private void stopPlayMusic(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        txAudioEffectManager.stopPlayMusic(id);
        result.success(null);
    }

    /**
     * 暂停播放背景音乐
     */
    private void pausePlayMusic(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        txAudioEffectManager.pausePlayMusic(id);
        result.success(null);
    }

    /**
     * 恢复播放背景音乐
     */
    private void resumePlayMusic(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        txAudioEffectManager.resumePlayMusic(id);
        result.success(null);
    }

    /**
     * 设置背景音乐的远端音量大小，即主播可以通过此接口设置远端观众能听到的背景音乐的音量大小。
     */
    private void setMusicPublishVolume(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        int volume = CommonUtil.getParam(call, result, "volume");
        txAudioEffectManager.setMusicPublishVolume(id, volume);
        result.success(null);
    }

    /**
     * 设置背景音乐的本地音量大小，即主播可以通过此接口设置主播自己本地的背景音乐的音量大小。
     */
    private void setMusicPlayoutVolume(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        int volume = CommonUtil.getParam(call, result, "volume");
        txAudioEffectManager.setMusicPlayoutVolume(id, volume);
        result.success(null);
    }

    /**
     * 设置全局背景音乐的本地和远端音量的大小
     */
    private void setAllMusicVolume(@NonNull MethodCall call, @NonNull Result result) {
        int volume = CommonUtil.getParam(call, result, "volume");
        txAudioEffectManager.setAllMusicVolume(volume);
        result.success(null);
    }

    /**
     * 调整背景音乐的音调高低
     */
    private void setMusicPitch(@NonNull MethodCall call, @NonNull Result result) {
        int volume = CommonUtil.getParam(call, result, "volume");
        String pitchParam = CommonUtil.getParam(call, result, "pitch");
        float pitch = Float.parseFloat(pitchParam);
        txAudioEffectManager.setMusicPitch(volume, pitch);
        result.success(null);
    }

    /**
     * 调整背景音乐的变速效果
     */
    private void setMusicSpeedRate(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        String speedRateParam = CommonUtil.getParam(call, result, "speedRate");
        float speedRate = Float.parseFloat(speedRateParam);
        txAudioEffectManager.setMusicSpeedRate(id, speedRate);
        result.success(null);
    }

    /**
     * 获取背景音乐当前的播放进度（单位：毫秒）
     */
    private void getMusicCurrentPosInMS(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        result.success(txAudioEffectManager.getMusicCurrentPosInMS(id));
    }

    /**
     * 设置背景音乐的播放进度（单位：毫秒）
     */
    private void seekMusicToPosInMS(@NonNull MethodCall call, @NonNull Result result) {
        int id = CommonUtil.getParam(call, result, "id");
        int pts = CommonUtil.getParam(call, result, "pts");
        txAudioEffectManager.seekMusicToPosInMS(id, pts);
        result.success(null);
    }

    /**
     * 获取景音乐文件的总时长（单位：毫秒）
     */
    private void getMusicDurationInMS(@NonNull MethodCall call, @NonNull Result result) {
        String path = CommonUtil.getParam(call, result, "path");
        result.success(txAudioEffectManager.getMusicDurationInMS(path));
    }

    /**
     * 生成用户签名。
     */
    private void genTestUserSig(@NonNull MethodCall call, @NonNull Result result) {
        int sdkAppId = CommonUtil.getParam(call, result, "sdkAppId");
        String secretkey = CommonUtil.getParam(call, result, "secretKey");
        String userId = CommonUtil.getParam(call, result, "userId");
        int expireTime = CommonUtil.getParam(call, result, "expireTime");
        result.success(GenerateTestUserSig.genTestUserSig(sdkAppId, secretkey, userId, expireTime));
    }
}