import 'dart:async';
import 'dart:convert';
import 'package:flutter/services.dart';
import 'TRTCCloudListener.dart';
import 'TRTCCloudDef.dart';
import 'TXBeautyManager.dart';
import 'TXAudioEffectManager.dart';
import 'TXDeviceManager.dart';
import 'util/CommonUtil.dart';

class TRTCCloud {
  static TRTCCloud _trtcCloud;

  static const MethodChannel _channel = const MethodChannel('trtcCloudChannel');

  static TRTCCloudListener listener;

  /// 创建 TRTCCloud 单例。
  static Future<TRTCCloud> sharedInstance() async {
    if (_trtcCloud == null) {
      _trtcCloud = new TRTCCloud();
      await _channel.invokeMethod('sharedInstance');
    }
    return _trtcCloud;
  }

  /// 销毁 TRTCCloud 单例。
  static Future<void> destroySharedInstance() async {
    await _channel.invokeMethod('destroySharedInstance');
    _trtcCloud = null;
  }

  /// 添加消息监听。
  void setListener(ListenerValue func) {
    if (listener == null) {
      listener = TRTCCloudListener(_channel);
    }
    listener.addListener(func);
  }

  /// 移除消息监听。
  void removeListener(ListenerValue func) {
    if (listener == null) {
      listener = TRTCCloudListener(_channel);
    }
    listener.removeListener(func);
  }

  /// 进入房间，若房间不存在，系统将自动创建一个新房间。
  Future<void> enterRoom(TRTCParams param, int scene) {
    return _channel.invokeMethod('enterRoom', {
      "param": jsonEncode(param),
      "scene": scene,
    });
  }

  /// 离开房间。
  ///
  /// 调用 exitRoom() 接口会执行退出房间的相关逻辑，例如释放音视频设备资源和编解码器资源等。 待资源释放完毕，SDK 会通过 onExitRoom() 回调通知到您。
  ///
  /// 如果您要再次调用 enterRoom() 或者切换到其他的音视频 SDK，请等待 onExitRoom() 回调到来之后再执行相关操作。 否则可能会遇到摄像头或麦克风被占用等各种异常问题，例如常见的 Android 媒体音量和通话音量切换问题等等。
  Future<void> exitRoom() {
    return _channel.invokeMethod('exitRoom');
  }

  /// 切换角色，仅适用于直播场景（TRTC_APP_SCENE_LIVE 和 TRTC_APP_SCENE_VOICE_CHATROOM）。
  ///
  /// 在直播场景下，一个用户可能需要在“观众”和“主播”之间来回切换。 您可以在进房前通过 TRTCParams 中的 role 字段确定角色，也可以通过 switchRole 在进房后切换角色。
  Future<void> switchRole(int role // 目标角色，默认为主播
      ) {
    return _channel.invokeMethod('switchRole', {
      "role": role,
    });
  }

  /// 设置音视频数据接收模式（需要在进房前设置才能生效）。
  Future<void> setDefaultStreamRecvMode(
      bool
          autoRecvAudio, // true：自动接收音频数据；false：需要调用 muteRemoteAudio 进行请求或取消。默认值：true。
      bool
          autoRecvVideo // true：自动接收视频数据；false：需要调用 startRemoteView/stopRemoteView 进行请求或取消。默认值：true。
      ) {
    return _channel.invokeMethod('setDefaultStreamRecvMode', {
      "autoRecvAudio": autoRecvAudio,
      "autoRecvVideo": autoRecvVideo,
    });
  }

  // CDN 相关接口函数

  /// 设置云端的混流转码参数。
  Future<void> setMixTranscodingConfig(TRTCTranscodingConfig config) {
    return _channel.invokeMethod('setMixTranscodingConfig', {
      "config": jsonEncode(config),
    });
  }

  // 视频相关接口函数

  /// 暂停/恢复推送本地的视频数据。
  Future<void> muteLocalVideo(bool mute // true：屏蔽；false：开启，默认值：false。
      ) {
    return _channel.invokeMethod('muteLocalVideo', {
      "mute": mute,
    });
  }

  /// 停止本地视频采集及预览。
  Future<void> stopLocalPreview() {
    return _channel.invokeMethod('stopLocalPreview');
  }

  /// 停止显示远端视频画面，同时不再拉取该远端用户的视频数据流。
  ///
  /// 指定要停止观看的 userId 的视频流类型：
  ///*     - 高清大画面：TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG
  ///*     - 低清大画面：TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SMALL
  ///*     - 辅流（屏幕分享）：TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB
  Future<void> stopRemoteView(
      String userId, // 用户ID
      int streamType) {
    return _channel.invokeMethod(
        'stopRemoteView', {"userId": userId, "streamType": streamType});
  }

  /// 停止显示所有远端视频画面，同时不再拉取远端用户的视频数据流。
  Future<void> stopAllRemoteView() {
    return _channel.invokeMethod('stopAllRemoteView');
  }

  /// 暂停/恢复接收指定的远端视频流。
  Future<void> muteRemoteVideoStream(
      userId, // 用户ID
      bool mute // 是否停止接收
      ) {
    return _channel.invokeMethod('muteRemoteVideoStream', {
      "userId": userId,
      "mute": mute,
    });
  }

  /// 暂停/恢复接收所有远端视频流。
  Future<void> muteAllRemoteVideoStreams(bool mute // 是否停止接收
      ) {
    return _channel.invokeMethod('muteAllRemoteVideoStreams', {
      "mute": mute,
    });
  }

  /// 设置视频编码器相关参数。
  Future<void> setVideoEncoderParam(
      TRTCVideoEncParam
          param // 视频编码参数，详情请参考 TRTCCloudDef.java 中的 TRTCVideoEncParam 定义。
      ) {
    return _channel.invokeMethod('setVideoEncoderParam', {
      "param": jsonEncode(param),
    });
  }

  /// 设置网络流控相关参数。
  ///
  /// 该设置决定 SDK 在各种网络环境下的调控策略（例如弱网下选择“保清晰”或“保流畅”）
  Future<void> setNetworkQosParam(
      TRTCNetworkQosParam
          param // 网络流控参数，详情请参考 TRTCCloudDef.java 中的 TRTCNetworkQosParam 定义
      ) {
    return _channel
        .invokeMethod('setNetworkQosParam', {"param": jsonEncode(param)});
  }

  /// 设置本地图像的渲染模式。
  ///
  /// @param renderParams 渲染参数（平铺模式、旋转角度、左右镜像等)，详见 {@link TRTCCloudDef#TRTCRemoteRenderParams}
  Future<void> setLocalRenderParams(TRTCRenderParams renderParams) {
    return _channel.invokeMethod('setLocalRenderParams', {
      "param": jsonEncode(renderParams),
    });
  }

  /// 设置远端图像相关参数。
  ///
  /// @param userId 用户 ID
  ///* @param streamType 视频流类型
  ///*     - 高清大画面：TRTCCloudDef#TRTC_VIDEO_STREAM_TYPE_BIG
  ///*     - 低清大画面：TRTCCloudDef#TRTC_VIDEO_STREAM_TYPE_SMALL
  ///*     - 辅流（屏幕分享）：TRTCCloudDef#TRTC_VIDEO_STREAM_TYPE_SUB
  ///* @param renderParams 渲染参数（平铺模式、旋转角度、左右镜像等)，详见 {@link TRTCCloudDef#TRTCRemoteRenderParams}
  Future<void> setRemoteRenderParams(
      String userId, int streamType, TRTCRenderParams renderParams) {
    return _channel.invokeMethod('setRemoteRenderParams', {
      "userId": userId,
      "streamType": streamType,
      "param": jsonEncode(renderParams),
    });
  }

  /// 设置视频编码输出的画面方向，即设置远端用户观看到的和服务器录制的画面方向。
  Future<void> setVideoEncoderRotation(
      int rotation // 目前支持 TRTC_VIDEO_ROTATION_0 和 TRTC_VIDEO_ROTATION_180 两个旋转角度，默认值：TRTC_VIDEO_ROTATION_0。
      ) {
    return _channel.invokeMethod('setVideoEncoderRotation', {
      "rotation": rotation,
    });
  }

  /// 设置编码器输出的画面镜像模式。
  ///
  /// 该接口不改变本地摄像头的预览画面，但会改变另一端用户看到的（以及服务器录制下来的）画面效果。
  Future<void> setVideoEncoderMirror(bool mirror // true：镜像；false：不镜像；默认值：false。
      ) {
    return _channel.invokeMethod('setVideoEncoderMirror', {
      "mirror": mirror,
    });
  }

  /// 设置重力感应的适应模式。
  Future<void> setGSensorMode(
      int mode // 重力感应模式，详情请参考 TRTC_GSENSOR_MODE 的定义，默认值：TRTC_GSENSOR_MODE_UIFIXLAYOUT。
      ) {
    return _channel.invokeMethod('setGSensorMode', {
      "mode": mode,
    });
  }

  /// 开启大小画面双路编码模式。
  Future<void> enableEncSmallVideoStream(
      bool enable, // 是否开启小画面编码，默认值：false。
      TRTCVideoEncParam smallVideoEncParam // 小流的视频参数。
      ) {
    return _channel.invokeMethod('enableEncSmallVideoStream', {
      "enable": enable,
      "smallVideoEncParam": smallVideoEncParam.toJson(),
    });
  }

  /// 选定观看指定 uid 的大画面或小画面。
  Future<void> setRemoteVideoStreamType(
      String userId, // 用户ID
      int streamType // 视频流类型，即选择看大画面或小画面，默认为大画面。
      ) {
    return _channel.invokeMethod('setRemoteVideoStreamType', {
      "userId": userId,
      "streamType": streamType,
    });
  }

  /// 设定观看方优先选择的视频质量。
  Future<void> setPriorRemoteVideoStreamType(
      int streamType // 默认观看大画面或小画面，默认为大画面。
      ) {
    return _channel.invokeMethod('setPriorRemoteVideoStreamType', {
      "streamType": streamType,
    });
  }

  /// 视频画面截图。
  ///
  /// 【特别说明】该路径需精确到文件名及格式后缀，格式后缀决定图片的格式，目前支持的格式有 png/jpf/webp。 例如，指定路径为 path/to/test.png，则会生成一个 png 格式的图片文件。 请指定一个有读写权限的合法路径，否则图片文件无法生成。
  Future<void> snapshotVideo(
    String
        userId, // 用户 ID，null 表示截取本地视频画面，本地仅支持摄像头画面（TRTCCloudDef#TRTC_VIDEO_STREAM_TYPE_BIG）的截取。

    int streamType, // 视频流类型，支持摄像头画面（TRTCCloudDef#TRTC_VIDEO_STREAM_TYPE_BIG）和 屏幕分享画面（TRTCCloudDef#TRTC_VIDEO_STREAM_TYPE_SUB）。
    String path, // 截图后保存的图片地址
  ) {
    return _channel.invokeMethod('snapshotVideo',
        {"userId": userId, "streamType": streamType, "path": path});
  }

  // 音频相关接口函数

  /// 开启本地音频的采集和上行,并设置音频质量。
  ///
  /// 主播端的音质越高，观众端的听感越好，但传输所依赖的带宽也就越高，在带宽有限的场景下也更容易出现卡顿。
  Future<void> startLocalAudio(int quality) {
    return _channel.invokeMethod('startLocalAudio', {"quality": quality});
  }

  /// 关闭本地音频的采集和上行。
  Future<void> stopLocalAudio() {
    return _channel.invokeMethod('stopLocalAudio');
  }

  /// 静音/取消静音本地的音频。
  ///
  /// 当静音本地音频后，房间里的其它成员会收到 onUserAudioAvailable(userId, false) 回调通知。 当取消静音本地音频后，房间里的其它成员会收到 onUserAudioAvailable(userId, true) 回调通知。
  ///
  /// 与 stopLocalAudio 不同之处在于，muteLocalAudio(true) 并不会停止发送音视频数据，而是继续发送码率极低的静音包。 由于 MP4 等视频文件格式，对于音频的连续性是要求很高的，使用 stopLocalAudio 会导致录制出的 MP4 不易播放。 因此在对录制质量要求很高的场景中，建议选择 muteLocalAudio，从而录制出兼容性更好的 MP4 文件。
  Future<void> muteLocalAudio(bool mute // true：屏蔽；false：开启，默认值：false。
      ) {
    return _channel.invokeMethod('muteLocalAudio', {
      "mute": mute,
    });
  }

  /// 设置音频路由。
  Future<void> setAudioRoute(int route // 音频路由，即声音由哪里输出（扬声器、听筒）
      ) {
    return _channel.invokeMethod('setAudioRoute', {
      "route": route,
    });
  }

  /// 静音/取消静音指定的远端用户的声音。
  Future<void> muteRemoteAudio(
      String userId, // 对方的用户 ID
      bool mute // true静音，false静音
      ) {
    return _channel.invokeMethod('muteRemoteAudio', {
      "userId": userId,
      "mute": mute,
    });
  }

  /// 静音/取消静音所有用户的声音。
  Future<void> muteAllRemoteAudio(bool mute // true静音，false静音
      ) {
    return _channel.invokeMethod('muteAllRemoteAudio', {
      "mute": mute,
    });
  }

  /// 设置某个远程用户的播放音量
  Future<void> setRemoteAudioVolume(
      String userId, // 远程用户 ID
      int volume // 音量大小，取值0 - 100
      ) {
    return _channel.invokeMethod('setRemoteAudioVolume', {
      "userId": userId,
      "volume": volume,
    });
  }

  /// 设置 SDK 采集音量。
  Future<void> setAudioCaptureVolume(int volume // volume	音量大小，取值0 - 100，默认值为100
      ) {
    return _channel.invokeMethod('setAudioCaptureVolume', {
      "volume": volume,
    });
  }

  /// 获取 SDK 采集音量。
  Future<int> getAudioCaptureVolume() {
    return _channel.invokeMethod('getAudioCaptureVolume');
  }

  /// 设置 SDK 播放音量。
  Future<void> setAudioPlayoutVolume(int volume // volume	音量大小，取值0 - 100，默认值为100
      ) {
    return _channel.invokeMethod('setAudioPlayoutVolume', {
      "volume": volume,
    });
  }

  /// 获取 SDK 播放音量
  Future<int> getAudioPlayoutVolume() {
    return _channel.invokeMethod('getAudioPlayoutVolume');
  }

  /// 启用音量大小提示。
  Future<void> enableAudioVolumeEvaluation(
      int intervalMs // 决定了 onUserVoiceVolume 回调的触发间隔，单位为ms，最小间隔为100ms，如果小于等于0则会关闭回调，建议设置为300ms；详细的回调规则请参考 onUserVoiceVolume 的注释说明。
      ) {
    return _channel.invokeMethod('enableAudioVolumeEvaluation', {
      "intervalMs": intervalMs,
    });
  }

  /// 开始录音。
  ///
  /// 该方法调用后， SDK 会将通话过程中的所有音频（包括本地音频，远端音频，BGM 等）录制到一个文件里。 无论是否进房，调用该接口都生效。 如果调用 exitRoom 时还在录音，录音会自动停止。
  Future<void> startAudioRecording(
      String filePath // 文件路径（必填），录音的文件地址，由用户自行指定，请确保 App 里指定的目录存在且可写。
      ) {
    return _channel.invokeMethod('startAudioRecording', {
      "filePath": filePath,
    });
  }

  /// 停止录音。
  Future<void> stopAudioRecording() {
    return _channel.invokeMethod('stopAudioRecording');
  }

  /// 设置通话时使用的系统音量类型。
  Future<void> setSystemVolumeType(
      int type // 系统音量类型，请参考 TRTCSystemVolumeType，默认值：TRTCSystemVolumeTypeAuto。
      ) {
    return _channel.invokeMethod('setSystemVolumeType', {
      "type": type,
    });
  }

  /// 开启耳返。支持Android，不支持iOS。
  Future<void> enableAudioEarMonitoring(bool enable // true：开启；false：关闭。
      ) {
    return _channel.invokeMethod('enableAudioEarMonitoring', {
      "enable": enable,
    });
  }

  /// 获取设备管理模块。
  getDeviceManager() {
    _channel.invokeMethod('getDeviceManager');
    return new TXDeviceManager(_channel);
  }

  /// 获取美颜管理对象。
  getBeautyManager() {
    _channel.invokeMethod('getBeautyManager');
    return new TXBeautyManager(_channel);
  }

  /// 添加水印
  ///
  /// 水印的位置是通过 x, y, width 来指定的
  Future<void> setWatermark(
      String assetUrl, //assets 中的资源地址
      int streamType,
      double x,
      double y,
      double width) async {
    String imageUrl = await CommonUtil.copyAssetToLocal(assetUrl);
    return _channel.invokeMethod('setWatermark', {
      "imageUrl": imageUrl,
      "streamType": streamType,
      "x": x.toString(),
      "y": y.toString(),
      "width": width.toString()
    });
  }

  /// 音乐特效和人声特效

  /// 获取音效管理类 TXAudioEffectManager。
  getAudioEffectManager() {
    _channel.invokeMethod('getAudioEffectManager');
    return new TXAudioEffectManager(_channel);
  }

  // 网络测试

  /// 开始进行网络测速（视频通话期间请勿测试，以免影响通话质量）
  ///
  /// 测速结果将会用于优化 SDK 接下来的服务器选择策略，因此推荐您在用户首次通话前先进行一次测速，这将有助于我们选择最佳的服务器。 同时，如果测试结果非常不理想，您可以通过醒目的 UI 提示用户选择更好的网络。 测试结果通过 TRTCCloudListener.onSpeedTest 回调出来。
  Future<void> startSpeedTest(
      int sdkAppId, // 应用标识
      String userId, // 用户标识
      String userSig // 用户签名
      ) {
    return _channel.invokeMethod('startSpeedTest', {
      "sdkAppId": sdkAppId,
      "userId": userId,
      "userSig": userSig,
    });
  }

  /// 停止服务器测速。
  Future<void> stopSpeedTest() {
    return _channel.invokeMethod('stopSpeedTest');
  }

  // Log相关接口函数

  /// 获取 SDK 版本信息
  Future<String> getSDKVersion() {
    return _channel.invokeMethod('getSDKVersion');
  }

  /// 设置 Log 输出级别
  Future<void> setLogLevel(
      int level // 请参见 TRTC_LOG_LEVEL，默认值：TRTCCloudDef#TRTC_LOG_LEVEL_NULL
      ) {
    return _channel.invokeMethod('setLogLevel', {"level": level});
  }

  /// 启用或禁用控制台日志打印
  Future<void> setConsoleEnabled(bool enabled // 是否启用
      ) {
    return _channel.invokeMethod('setConsoleEnabled', {
      "enabled": enabled,
    });
  }

  /// 显示仪表盘
  ///
  /// 仪表盘是状态统计和事件消息浮层　view，方便调试。
  Future<void> showDebugView(int showType // 0：不显示；1：显示精简版；2：显示全量版，默认为不显示
      ) {
    return _channel.invokeMethod('showDebugView', {
      "mode": showType,
    });
  }

  /// 生成用户签名。
  static Future<String> genTestUserSig(
      int sdkAppId, String secretKey, String userId, int expireTime) {
    return _channel.invokeMethod('genTestUserSig', {
      "sdkAppId": sdkAppId,
      "secretKey": secretKey,
      "userId": userId,
      "expireTime": expireTime
    });
  }
}
