import 'dart:convert';
import 'package:flutter/services.dart';

/// 监听器类型枚举
enum ListenerTypeEnum {
  /// SDK加载错误
  onError,

  /// 警告，例如出现卡顿或者可恢复的解码失败。
  onWarning,

  /// 加入房间
  onEnterRoom,

  /// 退出房间
  onExitRoom,

  /// 切换角色
  onSwitchRole,

  /// 有用户加入当前房间。
  onRemoteUserEnterRoom,

  /// 有用户离开当前房间。
  onRemoteUserLeaveRoom,

  /// 用户是否上行视频数据
  onUserVideoAvailable,

  /// 用户是否上行屏幕数据
  onUserSubStreamAvailable,

  /// 用户是否上行音频数据
  onUserAudioAvailable,

  /// 开始渲染本地或远程用户的首帧画面。
  onFirstVideoFrame,

  /// 开始播放远程用户的首帧音频（本地声音暂不通知）。
  onFirstAudioFrame,

  /// 首帧本地音频数据已经被送出。
  onSendFirstLocalVideoFrame,

  /// 首帧本地视频数据已经被送出。
  onSendFirstLocalAudioFrame,

  /// 网络质量：该回调每2秒触发一次，统计当前网络的上行和下行质量。
  onNetworkQuality,

  /// 技术指标统计回调:。
  onStatistics,

  /// 跟服务器断开
  onConnectionLost,

  /// SDK 尝试重新连接到服务器。
  onTryToReconnect,

  /// SDK 跟服务器的连接恢复。
  onConnectionRecovery,

  /// 服务器测速的回调，SDK 对多个服务器 IP 做测速，每个 IP 的测速结果通过这个回调通知。
  onSpeedTest,

  /// 摄像头准备就绪。
  onCameraDidReady,

  /// 麦克风准备就绪
  onMicDidReady,

  /// 用于提示音量大小的回调，包括每个 userId 的音量和远端总音量。
  onUserVoiceVolume,

  /// 设置云端的混流转码参数的回调，对应于 TRTCCloud 中的 setMixTranscodingConfig() 接口。
  onSetMixTranscodingConfig,

  /// 播放音效结束回调。
  onAudioEffectFinished,

  ///背景音乐开始播放,需先调用setMusicObserver才会触发事件回调
  onMusicObserverStart,

  ///背景音乐的播放进度,需先调用setMusicObserver才会触发事件回调
  onMusicObserverPlayProgress,

  ///背景音乐已播放完毕,需先调用setMusicObserver才会触发事件回调
  onMusicObserverComplete,

  ///截图完成时回调
  onSnapshotComplete
}

/// 监听器对象
class TRTCCloudListener {
  /// 监听器列表
  Set<ListenerValue> listeners = Set();

  TRTCCloudListener(MethodChannel channel) {
    // 绑定监听器
    channel.setMethodCallHandler((methodCall) async {
      // 解析参数
      Map<String, dynamic> arguments = jsonDecode(methodCall.arguments);

      switch (methodCall.method) {
        case 'onListener':
          // 获得原始类型和参数
          String typeStr = arguments['type'];
          var params = arguments['params'];

          // 封装回调类型和参数
          ListenerTypeEnum type;

          // 初始化类型
          for (var item in ListenerTypeEnum.values) {
            if (item.toString().replaceFirst("ListenerTypeEnum.", "") ==
                typeStr) {
              type = item;
              break;
            }
          }

          // 没有找到类型就返回
          if (type == null) {
            throw MissingPluginException();
          }

          // 回调触发
          for (var item in listeners) {
            item(type, params);
          }

          break;
        default:
          throw MissingPluginException();
      }
    });
  }

  /// 添加消息监听
  void addListener(ListenerValue func) {
    listeners.add(func);
  }

  /// 移除消息监听
  void removeListener(ListenerValue func) {
    listeners.remove(func);
  }
}

/// 监听器值模型
typedef ListenerValue<P> = void Function(ListenerTypeEnum type, P params);
